from sobeit.imgui import ImGui, ImBool

from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.raknet import *
from modsa_lib.core.handlers import RPCHandler, HandlersManagementMixin

from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_rpc, build_rpc
from modsa_lib.core.player import PlayerIter


class ChatModule(ConfigMixin, HandlersManagementMixin):
    config_path = "modsa_basic/chat_config.json"

    def __init__(self):
        self.handlers = [RPCHandler(RPC_ClientMessage, self.event_message)]
        self.load_config()
        if self.config.get('chat_id_enabled', True):
            self.start()

    def get_default_config(self):
        return {'chat_id_enabled': True}

    @staticmethod
    def event_message(bs_data, packet_id):
        data = parse_rpc(bs_data, packet_id)
        text = data["text"]

        text_updated = False

        for player in PlayerIter():
            if (
                text.find(player.name) != -1
                and text.find(f'[{player.id}]') == -1
                and text.find(f'({player.id})') == -1
                and text.find(f'[ID: {player.id}]') == -1
            ):
                text = text.replace(player.name, f'{player.name} [{player.id}]')
                text_updated = True

        if text_updated:
            data["text"] = text
            return build_rpc(RPC_ClientMessage, data)


chat_module = ChatModule()


def render_chat_config(rendering: bool):
    if rendering:
        ImGuiUtils.header(" --- [Чатид] ---")
        imgui_chat_id_enabled = ImBool(chat_module.config.get('chat_id_enabled', True))
        if ImGui.Checkbox("Включить чатид", imgui_chat_id_enabled):
            chat_module.config["chat_id_enabled"] = imgui_chat_id_enabled.v
            chat_module.start() if imgui_chat_id_enabled.v else chat_module.stop()
    else:
        chat_module.config.save()
