from sobeit.raknet import BitStream

from modsa_lib.consts.raknet import *
from modsa_lib.utils.common import ColorUtils, TextUtils


def parse_rpc(bs_data, rpc_id) -> dict:
    if rpc_id == RPC_ServerJoin:
        playerid = bs_data.Read(TYPE_UINT16)
        d3d9_color = bs_data.Read(TYPE_UINT32)
        is_npc = bs_data.Read(TYPE_UINT8)
        name_len = bs_data.Read(TYPE_UINT8)
        name = TextUtils.bytes_to_text(bs_data.Read(TYPE_STRING, name_len))
        color = ColorUtils.d3d9color_to_hex(d3d9_color)
        return {"id": playerid, "color": color[0] + color[1] + color[2], "is_npc": is_npc, "name": name}
    elif rpc_id == RPC_ServerQuit:
        return {"id": bs_data.Read(TYPE_UINT16), "reason": bs_data.Read(TYPE_UINT8)}
    elif rpc_id == RPC_ClientMessage:
        d3d9_color = bs_data.Read(TYPE_UINT32)
        message_len = bs_data.Read(TYPE_INT32)
        message = TextUtils.bytes_to_text(bs_data.Read(TYPE_STRING, message_len))
        color = ColorUtils.d3d9color_to_hex(d3d9_color)
        return {"text": message, "color": color[0] + color[1] + color[2]}
    elif rpc_id == RPC_ServerCommand:
        message_len = bs_data.Read(TYPE_UINT32)
        message = bs_data.Read(TYPE_STRING, message_len)
        return {"text": TextUtils.bytes_to_text(message)}
    elif rpc_id == RPC_Chat:
        message_len = bs_data.Read(TYPE_UINT8)
        message = bs_data.Read(TYPE_STRING, message_len)
        return {"text": TextUtils.bytes_to_text(message)}
    elif rpc_id == RPC_ShowTextDraw:
        data = dict(position=[], rotation=[])
        data['id'] = bs_data.Read(TYPE_INT16)
        data['flags'] = bs_data.Read(TYPE_UINT8)
        data['letter_width'] = bs_data.Read(TYPE_FLOAT)
        data['letter_height'] = bs_data.Read(TYPE_FLOAT)
        data['letter_color'] = ColorUtils.d3d9color_to_hex(bs_data.Read(TYPE_UINT32), split=False)
        data['box_width'] = bs_data.Read(TYPE_FLOAT)
        data['box_height'] = bs_data.Read(TYPE_FLOAT)
        data['box_color'] = ColorUtils.d3d9color_to_hex(bs_data.Read(TYPE_UINT32), split=False)
        data['shadow'] = bs_data.Read(TYPE_UINT8)
        data['outline'] = bs_data.Read(TYPE_UINT8)
        data['background_color'] = ColorUtils.d3d9color_to_hex(bs_data.Read(TYPE_UINT32), split=False)
        data['style'] = bs_data.Read(TYPE_UINT8)
        data['selectable'] = bs_data.Read(TYPE_UINT8)
        data['position'].append(bs_data.Read(TYPE_FLOAT))
        data['position'].append(bs_data.Read(TYPE_FLOAT))
        data['model_id'] = bs_data.Read(TYPE_UINT16)
        data['rotation'].append(bs_data.Read(TYPE_FLOAT))
        data['rotation'].append(bs_data.Read(TYPE_FLOAT))
        data['rotation'].append(bs_data.Read(TYPE_FLOAT))
        data['zoom'] = bs_data.Read(TYPE_FLOAT)
        data['color'] = ColorUtils.d3d9color_to_hex(bs_data.Read(TYPE_UINT32), split=False)

        text_length = bs_data.Read(TYPE_UINT16)
        data['text'] = TextUtils.bytes_to_text(bs_data.Read(TYPE_STRING, text_length))
        return data
    elif rpc_id == RPC_TextDrawSetString:
        data = dict()
        data['id'] = bs_data.Read(TYPE_INT16)
        text_length = bs_data.Read(TYPE_UINT16)
        data['text'] = TextUtils.bytes_to_text(bs_data.Read(TYPE_STRING, text_length))
        return data
    elif rpc_id == RPC_TextDrawHideForPlayer or rpc_id == RPC_ClickTextDraw:
        return {'id': bs_data.Read(TYPE_INT16)}
    elif rpc_id == RPC_TogglePlayerSpectating:
        return {'state': bs_data.Read(TYPE_INT32)}


def build_rpc(rpc_id, data):
    bs_data = BitStream()

    if rpc_id == RPC_ClientMessage:
        d3d9_color = ColorUtils.hex_to_d3d9color(data['color'])
        message = TextUtils.text_to_bytes(data['text'])
        bs_data.Write(d3d9_color, TYPE_UINT32)
        bs_data.Write(len(message), TYPE_INT32)
        bs_data.Write(message, TYPE_STRING)

    elif rpc_id == RPC_ServerCommand:
        message = TextUtils.text_to_bytes(data['text'])
        bs_data.Write(len(message), TYPE_INT32)
        bs_data.Write(message, TYPE_STRING)

    elif rpc_id == RPC_Chat:
        message = TextUtils.text_to_bytes(data['text'])
        bs_data.Write(len(message), TYPE_UINT8)
        bs_data.Write(message, TYPE_STRING)

    elif rpc_id == RPC_ShowTextDraw:
        bs_data.Write(data['id'], TYPE_INT16)
        bs_data.Write(data['flags'], TYPE_UINT8)
        bs_data.Write(data['letter_width'], TYPE_FLOAT)
        bs_data.Write(data['letter_height'], TYPE_FLOAT)
        bs_data.Write(ColorUtils.hex_to_d3d9color(data['letter_color']), TYPE_UINT32)
        bs_data.Write(data['box_width'], TYPE_FLOAT)
        bs_data.Write(data['box_height'], TYPE_FLOAT)
        bs_data.Write(ColorUtils.hex_to_d3d9color(data['box_color']), TYPE_UINT32)
        bs_data.Write(data['shadow'], TYPE_UINT8)
        bs_data.Write(data['outline'], TYPE_UINT8)
        bs_data.Write(ColorUtils.hex_to_d3d9color(data['background_color']), TYPE_UINT32)
        bs_data.Write(data['style'], TYPE_UINT8)
        bs_data.Write(data['selectable'], TYPE_UINT8)
        bs_data.Write(data['position'][0], TYPE_FLOAT)
        bs_data.Write(data['position'][1], TYPE_FLOAT)
        bs_data.Write(data['model_id'], TYPE_UINT16)
        bs_data.Write(data['rotation'][0], TYPE_FLOAT)
        bs_data.Write(data['rotation'][1], TYPE_FLOAT)
        bs_data.Write(data['rotation'][2], TYPE_FLOAT)
        bs_data.Write(data['zoom'], TYPE_FLOAT)
        bs_data.Write(ColorUtils.hex_to_d3d9color(data['color']), TYPE_UINT32)

        text = TextUtils.text_to_bytes(data['text'])
        bs_data.Write(len(text), TYPE_UINT16)
        bs_data.Write(text, TYPE_STRING)

    elif rpc_id == RPC_TextDrawHideForPlayer or rpc_id == RPC_ClickTextDraw:
        bs_data.Write(data['id'], TYPE_INT16)

    return bs_data


def parse_packet(bs_data, packet_id, incomming=True):
    data = {}

    bs_data.IgnoreBits(8)
    if packet_id == ID_SPECTATOR_SYNC:
        data = {
            'lrAnalog': bs_data.Read(TYPE_INT16),
            'udAnalog': bs_data.Read(TYPE_INT16),
            'keys': bs_data.Read(TYPE_UINT16),
            'position': [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)],
        }
    elif packet_id == ID_PLAYER_SYNC:
        data = {
            'lrAnalog': 0,
            'udAnalog': 0,
        }

        if incomming:
            data['id'] = bs_data.Read(TYPE_INT16)
            if bs_data.Read(TYPE_BOOLEAN):
                data['lrAnalog'] = bs_data.Read(TYPE_INT16)
            if bs_data.Read(TYPE_BOOLEAN):
                data['udAnalog'] = bs_data.Read(TYPE_INT16)
        else:
            data['lrAnalog'] = bs_data.Read(TYPE_INT16)
            data['udAnalog'] = bs_data.Read(TYPE_INT16)

        data['keys'] = bs_data.Read(TYPE_UINT16)
        data['position'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['quaternion'] = list(bs_data.ReadNormQuat())
        byte_hp_armour = bs_data.Read(TYPE_INT8)
        hp = byte_hp_armour >> 4
        armour = byte_hp_armour & 0x0F

        hp = 100 if hp == 0x0F else hp * 7
        armour = 100 if armour == 0x0F else armour * 7

        data['health'] = hp
        data['armour'] = armour
        data['weapon'] = bs_data.Read(TYPE_INT8)
        data['special_action'] = bs_data.Read(TYPE_INT8)
        data['speed'] = list(bs_data.ReadVector())

        if bs_data.Read(TYPE_BOOLEAN):
            data['surfing_vehicle'] = bs_data.Read(TYPE_INT16)
            data['surfing_offset'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]

        if bs_data.Read(TYPE_BOOLEAN):
            data['animation_id'] = bs_data.Read(TYPE_INT16)
            data['animation_flags'] = bs_data.Read(TYPE_INT16)
    elif packet_id == ID_BULLET_SYNC:
        if incomming:
            data['id'] = bs_data.Read(TYPE_INT16)
        data['type'] = bs_data.Read(TYPE_UINT8)
        data['target_id'] = bs_data.Read(TYPE_INT16)
        data['origin'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['target'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['center'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['weapon'] = bs_data.Read(TYPE_UINT8)
    elif packet_id == ID_AIM_SYNC:
        if incomming:
            data['id'] = bs_data.Read(TYPE_INT16)
        data['cam_mode'] = bs_data.Read(TYPE_UINT8)
        data['cam_front'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['cam_pos'] = [bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT), bs_data.Read(TYPE_FLOAT)]
        data['aim_z'] = bs_data.Read(TYPE_FLOAT)
        data['cam_ext_zoom'] = bs_data.Read(TYPE_UINT8)
        data['weapon_state'] = bs_data.Read(TYPE_UINT8)
        data['unknown'] = bs_data.Read(TYPE_UINT8)

    return data
