from sobeit import chat_api

from modsa_lib.utils.common import TextUtils


class Chat:
    @classmethod
    def print_text(cls, text: str, color: str = "FFFFFF"):
        """
        Вывод сообщения в чат
        Args:
            text: текст сообщения
            color: цвет сообщения
        """
        chat_api.raw_print_text(TextUtils.text_to_bytes(text), color)

    @classmethod
    def send_text(cls, text: str, use_raknet: bool = True):
        """
        Отправка сообщения серверу
        Args:
            text: текст сообщения
            use_raknet: если True, сообщение не попадёт в обработчик RPC_ServerCommand
        """
        chat_api.raw_send_text(TextUtils.text_to_bytes(text), use_raknet)

    @classmethod
    def get_messages(cls, msg_count: int = 10, client_msg: bool = False) -> list:
        """
        Список сообщений из чата (начиная с последнего)
        Args:
            msg_count: макс. количество сообщений (не более 100)
            client_msg: включать сообщения клиента
        Return:
            Список словарей. Каждый словарь содержит поля text, color (в HEX) и time.
            Список начинается с самого последнего сообщения.
        """
        messages = chat_api.raw_get_messages(msg_count, client_msg)
        for message in messages:
            message['text'] = TextUtils.bytes_to_text(message['text'])

        return messages

    @classmethod
    def set_chat_block(cls, state: bool):
        """Блокировка открытия чата на клавишу T"""
        chat_api.set_chat_block(state)
