from sobeit.imgui import ImVec2, ImBool, ImGui
from typing import Optional

from modsa_lib.core import RenderAPI, Chat
from modsa_lib.imgui.flags import *


class ImguiWindow:
    def __init__(self, name, flags=0, pos: Optional[ImVec2] = None, size=None, disable_chat=False):
        """
        Args:
            name: заголовок окна
            flags: флаги (ImGuiWindowFlags)
            pos: начальное расположение (по умолчанию центр экрана)
            size: размер
            disable_chat: не открывать чат (при нажатии T) во время показа
        """
        self.name = name
        self.flags = flags
        self.start_pos = pos
        self.size = size
        self.disable_chat = disable_chat
        self._imgui_is_open = ImBool(False)

    def __enter__(self):
        self.before_render()
        ImGui.Begin(self.name, self._imgui_is_open, self.flags)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        ImGui.End()
        self.after_render()

    def is_open(self):
        return self._imgui_is_open.v

    def show(self):
        self._imgui_is_open.v = True
        if self.disable_chat:
            Chat.set_chat_block(True)

    def hide(self):
        self._imgui_is_open.v = False
        if self.disable_chat:
            Chat.set_chat_block(False)

    def before_render(self):
        if not self.start_pos:
            self.start_pos = ImVec2(int(RenderAPI.display_width * 0.5), int(RenderAPI.display_height * 0.5))
        ImGui.PushStyleVar(ImGuiStyleVar_WindowRounding, 12.0)
        ImGui.PushStyleVar(ImGuiStyleVar_FrameRounding, 6.0)
        if self.size:
            ImGui.SetNextWindowSize(self.size, ImGuiCond_FirstUseEver)
        if self.start_pos:
            ImGui.SetNextWindowPos(self.start_pos, pivot=ImVec2(0.5, 0.5))
            self.start_pos = None

    def after_render(self):
        ImGui.PopStyleVar(2)