from typing import Optional

from modsa_lib.utils.json_helper import JsonFile
from modsa_lib.settings import SCRIPT_SETTINGS_PATH


class Config:
    path = None
    json_file = None
    data = None

    def __init__(self, path: str, default_config: Optional[dict] = None, use_settings_path=True):
        self.config_path = f'{SCRIPT_SETTINGS_PATH}/{path}' if use_settings_path else path
        self.default_config = default_config if default_config else {}
        self.load()

    def __getitem__(self, key):
        return self.data.get(key)

    def __setitem__(self, key, value):
        self.data[key] = value

    def __iter__(self):
        for key in self.data.keys():
            yield key

    def get(self, key, def_value=None):
        return self.data.get(key, def_value)

    def load(self):
        self.data = self.default_config.copy()
        if self.config_path:
            self.json_file = JsonFile(self.config_path, default_data=self.data)
            self.data.update(self.json_file.data)

    def save(self):
        if self.config_path and self.data:
            self.json_file.write(self.data)


class ConfigMixin:
    """Миксин для работы с файлом конфигурации"""

    config_path = None
    config = None

    def load_config(self):
        if self.config_path is None:
            raise Exception('Не указан путь к файлу конфига')

        self.config = Config(self.config_path, self.get_default_config())

    def save_config(self):
        self.config.save()

    def get_default_config(self):
        return {}
