import re

from modsa_lib.core import Chat
from modsa_lib.utils.chat_queue import ChatQueue
from modsa_lib.core.command import CommandDocMixin, Command
from modsa_lib.consts.raknet import RPC_ClientMessage
from modsa_lib.core.handlers import HandlersManagementMixin, RPCHandler
from modsa_lib.core.bitstream import parse_rpc


chat_queue = ChatQueue()


class Ghere(CommandDocMixin, HandlersManagementMixin):
    def __init__(self):
        self.cmd_ghere = Command(
            'ghere',
            self.callback_ghere,
            description='массовая телепортация',
        )
        self.handlers = [RPCHandler(RPC_ClientMessage, self.event_message)]
        self.players_teleported = set()

    def callback_ghere(self, args, cmd):
        self.start() if not self.is_running() else self.stop()

    def start(self) -> bool:
        if not chat_queue.can_start():
            Chat.print_text(f"Запущен другой процесс")
            return False

        Chat.print_text("Запущено массовое ТП. Останвока: /ghere", "5096E6")
        chat_queue.start()
        return super().start()

    def stop(self) -> bool:
        chat_queue.stop()
        self.players_teleported.clear()
        Chat.print_text("Массовое ТП завершено", "5096E6")
        return super().stop()

    def event_message(self, bs_data, event):
        data = parse_rpc(bs_data, event)
        if data["text"].find(' SMS:') != 0:
            return

        text = data["text"]
        playerid = re.findall("\[(\d*)\]", text[-5:])

        if playerid and playerid[0] not in self.players_teleported:
            self.players_teleported.add(playerid[0])
            chat_queue.add(f'/gethere {playerid[0]}')

        return False


ghere_instance = Ghere()
