import threading
import time
import random

from modsa_lib.core import Chat
from modsa_lib.core.command import CommandDocMixin, Command, CmdArgParser, CmdArgInt
from modsa_lib.core.handlers import HandlersManagementMixin
from modsa_lib.core.player import PlayerIter, Player
from modsa_lib.utils.common import get_vector3_distance


class SharkEvent(CommandDocMixin, HandlersManagementMixin):
    def __init__(self):
        self.cmd_shark = Command(
            'shark',
            self.command_shark,
            alias=['акула'],
            description="МП \"Акула\". Игроки, которых коснётся администратор, умрут.",
        )

    def command_shark(self, args, cmd):
        self.start() if not self.is_running() else self.stop()

    def after_handlers_start(self):
        Chat.print_text("Запущено мероприятие 'Акула'. Остановка: /shark", "5096E6")
        t = threading.Thread(target=self.thread_shark, daemon=True)
        t.start()
        return super().start()

    def thread_shark(self):
        while self.is_running():
            time.sleep(0.2)
            players = PlayerIter().in_stream()
            local_position = Player().position
            for player in players:
                position = player.position
                if (
                    position
                    and get_vector3_distance(local_position, position) < 1
                    and player.health
                ):
                    Chat.send_text(f"/sethp {player['id']} 0")
                    time.sleep(1.1)

        Chat.print_text("Мероприятие 'Акула' завершено", "5096E6")


class SenoEvent(CommandDocMixin, HandlersManagementMixin):
    obj_start = -1
    obj_end = -1
    delay = 1

    def __init__(self):
        self.cmd_shark = Command(
            'mpseno',
            self.command_mpseno,
            alias=['сено'],
            description="МП \"Сено\"",
            arg_parser=CmdArgParser(
                {
                    'start': CmdArgInt(required=False, default=-1),
                    'end': CmdArgInt(required=False, default=-1),
                    'delay': CmdArgInt(required=False, default=1),
                }
            ),
        )

    def command_mpseno(self, args, cmd, parser_data):
        if not self.is_running():
            if parser_data['start'] == -1 or parser_data['start'] > parser_data['end']:
                Chat.print_text("Некорректные значения start и end", "5096E6")
                return
            self.obj_start = parser_data['start']
            self.obj_end = parser_data['end']
            self.delay = parser_data['delay']
            self.start()
        else:
            self.stop()

    def after_handlers_start(self):
        Chat.print_text("Запущено мероприятие 'Сено'. Остановка: /mpseno", "5096E6")
        t = threading.Thread(target=self.thread_seno, daemon=True)
        t.start()
        return super().start()

    def thread_seno(self):
        queue = list(range(self.obj_start, self.obj_end + 1))
        random.shuffle(queue)
        while self.is_running() and len(queue) > 1:
            time.sleep(self.delay)
            del_obj = queue.pop()
            Chat.send_text(f'/obj {del_obj} -1')

        self.stop()
        Chat.print_text("Мероприятие 'Сено' завершено", "5096E6")


shark_instance = SharkEvent()
seno_instance = SenoEvent()
