import re
import time

from sobeit import event_api
from sobeit.imgui import ImGui, ImBool, ImColor, ImVec2
from sobeit.raknet import BitStream

from modsa_basic.traces import tracer_manager, tracer_aim_manager
from modsa_lib.imgui.flags import *
from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.raknet import *
from modsa_lib.consts.sobeit import EVENT_GAME_INIT
from modsa_lib.core.event import Event
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_26, IMGUI_DEFAULT_FONT_23

# Шрифты
from modsa_lib.core.handlers import PacketHandler, RPCHandler, HandlersManagementMixin
from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_packet, parse_rpc
from modsa_lib.utils.recon import Recon
from modsa_lib.core.sobeit_api import RenderAPI

panel_font = IMGUI_DEFAULT_FONT_26
stats_font = IMGUI_DEFAULT_FONT_23


class ReconSrp(ConfigMixin, HandlersManagementMixin, Recon):
    config_path = 'srp_basic/recon.json'
    recon_textdraw_triggers = [2, 3, 4, 9, 10, 11] + list(range(52, 113))
    recon_td_player = -1
    recon_td_stats = -1

    def __init__(self):
        self.load_config()
        super().__init__(self.config.data['panel'], self.config.data['stats'])
        self.player_sync_handler = PacketHandler(ID_PLAYER_SYNC, self._player_sync_hook)
        self.pressed_player_buttons = {}
        self.pressed_window_pos = ImVec2(0, 0)
        self.color_not_pressed = ImColor(80, 80, 80)
        self.color_pressed = ImColor(80, 255, 80)
        self.render_traces = self.config.data['traces']
        self.traces_player = -1
        for x in [
            'w',
            's',
            'a',
            'd',
            'f',
            'lmb',
            'rmb',
            'alt',
            'c',
            'jump',
            'sprint',
            'tab',
        ]:
            self.pressed_player_buttons[x] = False

        if self.config.data['enabled']:
            self.handlers = [
                RPCHandler(RPC_ShowTextDraw, self.hook_textdraws),
                RPCHandler(RPC_TextDrawHideForPlayer, self.hook_textdraws),
                RPCHandler(RPC_TextDrawSetString, self.hook_textdraws),
                RPCHandler(RPC_ClickTextDraw, self.hook_textdraws),
                RPCHandler(RPC_TogglePlayerSpectating, self.hook_spectate),
            ]

            self.start()

    def _textdraw_click(self, textdraw_id: int):
        # Сдвиг ИДа текстдрава (Если сдвинулось после очередного обновления)
        # 28.06.21
        textdraw_id += 7

        spec_data = BitStream()
        spec_data.Write(ID_SPECTATOR_SYNC, TYPE_INT8)
        spec_data.Write(0, TYPE_INT16)
        spec_data.Write(0, TYPE_INT16)
        spec_data.Write(16, TYPE_INT16)
        spec_data.Write(50.0, TYPE_FLOAT)
        spec_data.Write(50.0, TYPE_FLOAT)
        spec_data.Write(50.0, TYPE_FLOAT)
        event_api.send_packet(spec_data)
        time.sleep(0.1)
        textdraw_data = BitStream()
        textdraw_data.Write(textdraw_id, TYPE_INT16)
        event_api.send_rpc(RPC_ClickTextDraw, textdraw_data)

    def show(self, change_mode=False) -> bool:
        changed = super().show(change_mode)
        if changed:
            self.player_sync_handler.start()

        return changed

    def hide(self) -> bool:
        changed = super().hide()
        if changed:
            self.player_sync_handler.stop()
            tracer_aim_manager.unwatch()

        return changed

    def render_stats(self):
        super().render_stats()
        ImGui.SetNextWindowPos(self.pressed_window_pos, ImGuiCond_FirstUseEver)
        ImGui.Begin(
            "##RECON_STATS_KEYS",
            ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoResize,
        )
        if self._change_mode:
            window_pos = ImGui.GetWindowPos()
            self.config.data["player_keys"]["x"] = window_pos.x
            self.config.data["player_keys"]["y"] = window_pos.y

        ImGui.TextColored(self._get_color_for_key('w'), 'W')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('s'), 'S')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('a'), 'A')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('d'), 'D')
        ImGui.TextColored(self._get_color_for_key('lmb'), 'ЛКМ')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('rmb'), 'ПКМ')
        ImGui.TextColored(self._get_color_for_key('jump'), 'Jump')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('c'), 'C')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('alt'), 'Alt')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('f'), 'F')
        ImGui.TextColored(self._get_color_for_key('sprint'), 'Sprint')
        ImGui.SameLine()
        ImGui.TextColored(self._get_color_for_key('tab'), 'Tab')
        ImGui.End()

    def _get_color_for_key(self, key: str):
        return self.color_pressed if self.pressed_player_buttons[key] else self.color_not_pressed

    def _player_sync_hook(self, bs_data, event):
        data = parse_packet(bs_data, event)
        if data['id'] == self.player_id:
            if data['lrAnalog'] == 128:
                self.pressed_player_buttons['d'] = True
                self.pressed_player_buttons['a'] = False
            elif data['lrAnalog'] == -128:
                self.pressed_player_buttons['d'] = False
                self.pressed_player_buttons['a'] = True
            else:
                self.pressed_player_buttons['d'] = False
                self.pressed_player_buttons['a'] = False

            if data['udAnalog'] == 128:
                self.pressed_player_buttons['w'] = False
                self.pressed_player_buttons['s'] = True
            elif data['udAnalog'] == -128:
                self.pressed_player_buttons['w'] = True
                self.pressed_player_buttons['s'] = False
            else:
                self.pressed_player_buttons['w'] = False
                self.pressed_player_buttons['s'] = False

            self.pressed_player_buttons['tab'] = bool(data['keys'] & 0b1)
            self.pressed_player_buttons['c'] = bool(data['keys'] >> 1 & 0b1)
            self.pressed_player_buttons['lmb'] = bool(data['keys'] >> 2 & 0b1)
            self.pressed_player_buttons['sprint'] = bool(data['keys'] >> 3 & 0b1)
            self.pressed_player_buttons['f'] = bool(data['keys'] >> 4 & 0b1)
            self.pressed_player_buttons['jump'] = bool(data['keys'] >> 5 & 0b1)
            self.pressed_player_buttons['rmb'] = bool(data['keys'] >> 7 & 0b1)
            self.pressed_player_buttons['alt'] = bool(data['keys'] >> 10 & 0b1)

    def hook_textdraws(self, bsdata, event):
        data = parse_rpc(bsdata, event)

        if event == RPC_ShowTextDraw:
            if data['position'] == [575.0, 210.0]:
                self.recon_td_stats = data['id']
            elif data['position'] == [585.0, 190.0]:
                self.recon_td_player = data['id']
        elif event == RPC_ClickTextDraw and self.is_rendering():
            if data['id'] == 213:
                click_data = BitStream()
                click_data.Write(-1, TYPE_INT16)
                event_api.send_rpc(RPC_ClickTextDraw, click_data)
        elif event == RPC_TextDrawSetString:
            if data['id'] == self.recon_td_stats:
                stats = data['text'].split('~n~')
                for index, element in enumerate(self.get_stats_items()):
                    try:
                        self.set_stats_data(element, stats[index + 1])
                    except IndexError:
                        self.set_stats_data(element, "<error>")
                        print(f"[Recon] Error: {element} not found.")
                        print(data['text'])

            elif data['id'] == self.recon_td_player:
                stats = data['text'].replace('~w~', '').replace('~r~', '').split('~n~')
                self.player_name = stats[0]
                playerid = re.findall("ID: (\d*)", stats[1])
                if playerid:
                    _playerid = int(playerid[0])
                    if self.render_traces:
                        if self.traces_player == -1:
                            self.traces_player = _playerid
                            tracer_manager.watch(_playerid)
                        elif self.traces_player != _playerid:
                            tracer_manager.unwatch(self.traces_player)
                            self.traces_player = _playerid
                            tracer_manager.watch(_playerid)
                    if self.config.data['aim']:
                        tracer_aim_manager.watch(_playerid)

                    self.player_id = _playerid
                else:
                    self.player_id = -1

        td_triggers = self.recon_textdraw_triggers + [
            self.recon_td_player,
            self.recon_td_stats,
        ]

        if event in [RPC_ShowTextDraw, RPC_TextDrawSetString, RPC_TextDrawHideForPlayer] and data['id'] in td_triggers:
            return False

    def hook_spectate(self, bsdata, event):
        data = parse_rpc(bsdata, event)
        if data['state'] == 1:
            self.show()
        else:
            self.hide()

    def render_settings(self, rendering: bool):

        if rendering:
            ImGuiUtils.header('--- [Recon] ---')
            self.show(True)
            imgui_recon_enabled = ImBool(self.config.data['enabled'])
            imgui_recon_traces = ImBool(self.config.data.get('traces', True))
            imgui_recon_aim = ImBool(self.config.data.get('aim', True))
            if ImGui.Checkbox("Включить новый рекон", imgui_recon_enabled):
                self.config.data['enabled'] = imgui_recon_enabled.v
                self.start() if imgui_recon_enabled.v else self.stop()
            if ImGui.Checkbox("Показывать трейсеры", imgui_recon_traces):
                self.config.data['traces'] = imgui_recon_traces.v
                self.render_traces = imgui_recon_traces.v
            if ImGui.Checkbox("Показывать AIM", imgui_recon_aim):
                self.config.data['aim'] = imgui_recon_aim.v
        else:
            self.hide()
            templates = self.get_templates()
            self.config.data['panel'] = templates['panel']
            self.config.data['stats'] = templates['stats']
            self.save_config()

    def get_default_config(self):
        return {
            "enabled": True,
            "traces": True,
            "aim": True,
            "panel": {
                "name": "recon_main",
                "size_x": 147.0,
                "size_y": 333.0,
                "pos_x": 5.0,
                "pos_y": 390.0,
                "items": {
                    "Change": 48,
                    "Check >>": {
                        "name": "recon_check",
                        "size_x": 150.0,
                        "size_y": 191.0,
                        "pos_x": 154.0,
                        "pos_y": 427.0,
                        "items": {
                            "GM": 66,
                            "GM2": 68,
                            "GMCar": 70,
                            "ResetShot": 72,
                            "ATipster": 74,
                        },
                    },
                    "Drop >>": {
                        "name": "recon_drop",
                        "size_x": 131.0,
                        "size_y": 191.0,
                        "pos_x": 155.0,
                        "pos_y": 461.0,
                        "items": {
                            "Mute": 76,
                            "Slap": 78,
                            "Prison": 80,
                            "Freeze": 82,
                            "Unfreeze": 84,
                        },
                    },
                    "Kick >>": {
                        "name": "recon_kick",
                        "size_x": 105.0,
                        "size_y": 88.0,
                        "pos_x": 156.0,
                        "pos_y": 495.0,
                        "items": {"Kick": 86, "SKick": 88},
                    },
                    "Warn": 56,
                    "Ban >>": {
                        "name": "recon_ban",
                        "size_x": 99.0,
                        "size_y": 122.0,
                        "pos_x": 156.0,
                        "pos_y": 566.0,
                        "items": {"Ban": 90, "SBan": 92, "IBan": 94},
                    },
                    "Stats >>": {
                        "name": "recon_stats",
                        "size_x": 115.0,
                        "size_y": 197.0,
                        "pos_x": 157.0,
                        "pos_y": 599.0,
                        "items": {
                            "Stats": 96,
                            "IWep": 98,
                            "GetIP": 100,
                            "Skills": 102,
                            "Satiety": 104,
                        },
                    },
                    "Refresh": 62,
                    "Exit": 64,
                },
            },
            "stats": {
                "size_x": 290.0,
                "size_y": 512.0,
                "pos_x": -320.0,
                "pos_y": 332.0,
                "items": {
                    "Level": {"color": "00AA22"},
                    "Warns": {"color": "00AA22"},
                    "Armour": {"color": "AA0000"},
                    "Health": {"color": "AA0000"},
                    "CarHP": {"color": "007799"},
                    "Speed": {"color": "007799"},
                    "Ping": {"color": "CCAA00"},
                    "Ammo": {"color": "CCAA00"},
                    "Shot": {"color": "DD00DD"},
                    "TimeShot": {"color": "DD00DD"},
                    "AFKTime": {"color": "00AA22"},
                    "Engine": {"color": "00AA22"},
                    "Frac": {"display": "Fct(Rank)", "color": "00AA22"},
                    "Mobile": {"display": "Mobile", "color": "00AA22"},
                },
            },
            "player_keys": {"x": -140, "y": 830},
        }


srp_recon = ReconSrp()
srp_recon.set_panel_font(panel_font)
srp_recon.set_stats_font(stats_font)


@Event.register(EVENT_GAME_INIT)
def on_game_init():
    srp_recon.pressed_window_pos = RenderAPI.parse_pos(
        srp_recon.config.data["player_keys"]["x"],
        srp_recon.config.data["player_keys"]["y"],
    )
