from sobeit.imgui import ImGui, ImColor

from modsa_lib.utils.checker import BaseChecker
from modsa_lib.consts.raknet import *
from modsa_lib.consts.sobeit import EVENT_GAME_INIT
from modsa_lib.core.event import Event
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_18
from modsa_lib.core.handlers import RPCHandler
from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_rpc
from modsa_lib.core.sobeit_api import PlayerAPI


class SQUADChecker(BaseChecker):
    config_path = 'srp_basic/squad_checker.json'
    default_direction_down = True
    window_font = IMGUI_DEFAULT_FONT_18

    def __init__(self):
        super().__init__()
        self.squad_rendering = False
        self.squad_td_id = None
        self.add_handlers(
            RPCHandler(RPC_ShowTextDraw, self.handle_td_rpc),
            RPCHandler(RPC_TextDrawSetString, self.handle_td_rpc),
            RPCHandler(RPC_TextDrawHideForPlayer, self.handle_td_rpc),
        )
        if self.config.data['enabled']:
            self.start()

    def render_title(self):
        ImGui.TextColored(ImColor(0, 180, 255), f"SQUAD ({self.online_count})")

    def pre_render(self):
        if self.squad_rendering:
            super().pre_render()

    def handle_td_rpc(self, bs_data, rpc_id):
        data = parse_rpc(bs_data, rpc_id)

        if not self.squad_td_id:
            if rpc_id == RPC_ShowTextDraw and data['position'] == [1.0, 172.0]:
                self.squad_td_id = data['id']
            else:
                return

        if data['id'] != self.squad_td_id:
            return

        if rpc_id == RPC_TextDrawHideForPlayer:
            self.squad_rendering = False
            self.clear_all_players()
        else:
            self.squad_rendering = True
            self.parse_squad_data(data['text'])
        return False

    def parse_squad_data(self, text: str):
        players = text.split('~n~')[1:]
        self.clear_all_players()
        for player in players:
            player_id = PlayerAPI.get_id(player)
            self.add_player(player, player_id)

    def get_default_config(self) -> dict:
        def_config = super().get_default_config()
        def_config.update(
            {
                'pos_x': 0.005,
                'pos_y': 0.5,
            }
        )
        return def_config

    def render_settings(self, rendering: bool):
        squad_checker.edit_mode = rendering

        if rendering:
            ImGuiUtils.header('--- [Squad чекер] ---')
            if ImGui.Checkbox("Включить##squad", squad_checker.imgui_config['enabled']):
                self.config.data['enabled'] = squad_checker.imgui_config['enabled'].v
                squad_checker.start() if squad_checker.imgui_config[
                    'enabled'
                ].v else squad_checker.stop()
            if ImGui.Checkbox(
                "Отображение сверху вниз##squad",
                squad_checker.imgui_config['direction_down'],
            ):
                self.config.data['direction_down'] = squad_checker.imgui_config[
                    'direction_down'
                ].v
            if ImGui.Checkbox(
                f"Динамическое изменение размера окна##squad",
                squad_checker.imgui_config['autosize'],
            ):
                self.config.data['autosize'] = squad_checker.imgui_config['autosize'].v
        else:
            self.save_config()


squad_checker = SQUADChecker()


@Event.register(EVENT_GAME_INIT)
def on_game_init():
    squad_checker.setup()
